package org.richfaces.demo.push;

import javax.faces.application.Application;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.PostConstructApplicationEvent;
import javax.faces.event.PreDestroyApplicationEvent;
import javax.faces.event.SystemEvent;
import javax.faces.event.SystemEventListener;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

public abstract class AbstractCapabilityInitializer implements SystemEventListener, ServletContextListener {

    private boolean correctlyInitialized = false;

    @Override
    public void processEvent(SystemEvent event) throws AbortProcessingException {
        if (isCapabilityEnabled()) {
            if (event instanceof PostConstructApplicationEvent) {
                Application application = FacesContext.getCurrentInstance().getApplication();
                application.subscribeToEvent(PreDestroyApplicationEvent.class, this);

                try {
                    initializeCapability();
                    correctlyInitialized = true;
                } catch (Exception e) {
                    throw new RuntimeException("Capability " + this.getClass().getName() + " was not correctly initialized", e);
                }
            } else {
                try {
                    finalizeCapability();
                } catch (Exception e) {
                    throw new RuntimeException("Capability " + this.getClass().getName() + " was not correctly finalized", e);
                }
            }
        }
    }
    
    @Override
    public boolean isListenerForSource(Object source) {
        return true;
    }

    @Override
    public void contextInitialized(ServletContextEvent sce) {
        try {
            initializeCapability();
            correctlyInitialized = true;
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    @Override
    public void contextDestroyed(ServletContextEvent sce) {
        try {
            finalizeCapability();
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }
    
    protected boolean isCorrentlyInitialized() {
        return correctlyInitialized;
    }
    
    public abstract void initializeCapability() throws Exception;
    public void finalizeCapability() throws Exception {}
    public boolean isCapabilityEnabled() { return true; }
}
